/* 
   Copyright: Copyright (c) 1998-2007 MOSEK ApS, Denmark. All rights reserved.

   File:      milo1.c

   Purpose:   To demonstrate how to solve a small mixed
              integer linear optimization problem using 
              the MOSEK API.
*/

#include <stdio.h>
 
#include "mosek.h" /* Include the MOSEK definition file. */

#define NUMCON 2   /* Number of constraints.             */
#define NUMVAR 2   /* Number of variables.               */
#define NUMANZ 4   /* Number of non-zeros in A.          */

static void MSKAPI printstr(void *handle,
                            char str[])
{
  printf("%s",str);
} /* printstr */

int main(int argc,char *argv[])
{
  MSKrescodee  r;
  double       c[]   = {  1.0, 0.64 };
  MSKboundkeye bkc[] = { MSK_BK_UP,    MSK_BK_LO };
  double       blc[] = { -MSK_INFINITY,-4.0 };
  double       buc[] = { 250.0,        MSK_INFINITY };

  MSKboundkeye bkx[] = { MSK_BK_LO,    MSK_BK_LO };
  double       blx[] = { 0.0,          0.0 };
  double       bux[] = { MSK_INFINITY, MSK_INFINITY };
  

  MSKlidxt     ptrb[] = { 0, 2 };
  MSKlidxt     ptre[] = { 2, 4 };
  MSKidxt      asub[] = { 0,    1,   0,    1 };
  double       aval[] = { 50.0, 3.0, 31.0, -2.0 };
  MSKidxt      j;

  double       xx[NUMVAR];
  MSKenv_t     env;
  MSKtask_t    task;

  /* Create the mosek environment. */
  r = MSK_makeenv(&env,NULL,NULL,NULL,NULL);

  /* Initialize the environment. */   
  if ( r==MSK_RES_OK )
    r = MSK_initenv(env);

  /* Check if return code is ok. */
  if ( r==MSK_RES_OK )
  {
    /* Directs the log stream to the 'printstr' function. */
    MSK_linkfunctoenvstream(env,MSK_STREAM_LOG,NULL,printstr);

    /* Create the optimization task. */
    r = MSK_maketask(env,NUMCON,NUMVAR,&task);

    if ( r==MSK_RES_OK )
      r = MSK_linkfunctotaskstream(task,MSK_STREAM_LOG,NULL,printstr);

    if ( r==MSK_RES_OK )
      r = MSK_inputdata(task,
                        NUMCON,NUMVAR,
                        NUMCON,NUMVAR,
                        c,0.0,
                        ptrb,
                        ptre,
                        asub,
                        aval,
                        bkc,
                        blc,
                        buc,
                        bkx,
                        blx,
                        bux);

    /* Specify integer variables. */
    for(j=0; j<NUMVAR && r == MSK_RES_OK; ++j)
      r = MSK_putvartype(task,j,MSK_VAR_TYPE_INT);
    
    if ( r==MSK_RES_OK )
      r =  MSK_putobjsense(task,
                           MSK_OBJECTIVE_SENSE_MAXIMIZE);

    if ( r==MSK_RES_OK )
      r = MSK_optimize(task);

    if ( r==MSK_RES_OK )
    {
      MSK_getsolutionslice(task,
                           /* Ask for integer solution */
                           MSK_SOL_ITG,
                           MSK_SOL_ITEM_XX,
                           0,
                           NUMVAR,
                           xx);

      printf("Primal solution\n");
      for(j=0; j<NUMVAR; ++j)
        printf("x[%d]: %e\n",j,xx[j]);
    }
  }
    
  MSK_deletetask(&task);
  MSK_deleteenv(&env);

  printf("Return code: %d.\n",r);

  return ( r );
} /* main */
