/*
  Copyright: Copyright (c) 1998-2007 MOSEK ApS, Denmark. All rights reserved.

  File:    testlic.c

  Purpose: Test if a MOSEK license is available. 
*/

#include "mosek.h"

static void MSKAPI printstr(void *handle,
                            char str[])
{
  printf("%s",str);
} /* printstr */


int main (int argc, char * argv[])
{
  MSKtask_t   task = NULL;
  MSKenv_t    env  = NULL;
  MSKrescodee r  = MSK_RES_OK;

  /* Make mosek environment. 
     The arguments which are here `NULL', are used to specify
     customized memory allocators and a memory debug file. These
     can safely be ignore for now. */
    
  r = MSK_makeenv(&env, NULL, NULL, NULL, NULL);

  /* Check if return code is ok. */
  if ( r==MSK_RES_OK )
  {
    /* Directs the env log stream to the user
       specified procedure 'printstr'. */
    MSK_linkfunctoenvstream(env,MSK_STREAM_LOG,NULL,printstr);
  }

  /* Initialize the environment */
  if ( r==MSK_RES_OK )
    MSK_initenv (env);

  /* Create a task object linked with the environment env.
     We create it initially with 0 variables and 0 columns, 
     since we don't know the size of the problem. */ 
  if ( r==MSK_RES_OK )
    r = MSK_maketask (env, 0,0, &task);

  if ( r==MSK_RES_OK )
    r = MSK_linkfunctotaskstream(task,MSK_STREAM_LOG,NULL,printstr);

  if (r == MSK_RES_OK)
    r = MSK_append(task,MSK_ACC_VAR,5000);
  if (r == MSK_RES_OK)
    r = MSK_append(task,MSK_ACC_CON,1);


  if (r == MSK_RES_OK)
    r = MSK_putintparam(task,MSK_IPAR_LICENSE_DEBUG,MSK_ON);
  
  if(r==MSK_RES_OK)
  {
    r = MSK_optimize(task);
  }

  MSK_deletetask(&task);
  MSK_deleteenv(&env);
  
  if (r == MSK_RES_OK)
  {
    printf("************************************\n");
    printf("A license was checked out correctly.\n");
    printf("************************************\n");
  }
  else
  {
    printf("************************************\n");
    printf("An error was encountered.\n");
    printf("The license could not be checked out.\n");
    printf("************************************\n");
  }
  

    
  return r;
}
