/*
   Copyright: Copyright (c) 1998-2007 MOSEK ApS, Denmark. All rights reserved.

   File:    mio1.cs

   Purpose:  Demonstrates how to solve a small mixed
             integer linear optimization problem using the MOSEK C# API.
 */


using System;

public class MsgClass : mosek.Stream
{

    public MsgClass ()
    {
      /* Construct the object */
    }

    public override void streamCB (string msg)
    {
        Console.Write ("{0}",msg);
    }
}

public class lo1
{
  
    public static void Main ()
    {
        const int NUMCON = 2;
        const int NUMVAR = 2;
        const int NUMANZ = 4;

        // Since the value infinity is never used, we define
        // 'infinity' symbolic purposes only
        double infinity = 0;

        
        mosek.boundkey[] bkc = { mosek.boundkey.up,
                                 mosek.boundkey.lo };
        double[] blc = { -infinity,
                         -4.0 };
        double[] buc = { 250.0,
                         infinity }; 

        mosek.boundkey[] bkx = { mosek.boundkey.lo,
                                 mosek.boundkey.lo  };
        double[] blx = { 0.0,
                         0.0 };
        double[] bux = { infinity,
                         infinity };

        double[] c   = {1.0, 0.64 };

        int[] asub    = {  0,   1,    0,    1   };
        double[] aval = { 50.0, 3.0, 31.0, -2.0 };

        int[] ptrb = { 0, 2 };
        int[] ptre = { 2, 4 };
        
        double[] xx  = new double[NUMVAR];

        mosek.Env env = null;
        mosek.Task task = null;

        try
          {
            // Make mosek environment. 
            env  = new mosek.Env ();
            // Direct the env log stream to the user specified
            // method env_msg_obj.streamCB 
            MsgClass env_msg_obj = new MsgClass ();
            env.set_streamCB (mosek.streamtype.log,env_msg_obj);
            // Initialize the environment.
            env.init ();
            // Create a task object linked with the environment env.
            task = new mosek.Task (env, NUMCON,NUMVAR);
            // Directs the log task stream to the user specified
            // method task_msg_obj.streamCB
            MsgClass task_msg_obj = new MsgClass ();
            task.set_streamCB (mosek.streamtype.log,task_msg_obj);
                
            /*TAG:begin-inputdata*/
            task.inputdata(NUMCON,NUMVAR,
                           c,
                           0.0,
                           ptrb,
                           ptre,
                           asub,
                           aval,
                           bkc,
                           blc,
                           buc,
                           bkx,
                           blx,
                           bux);
            
            /* Specify integer variables. */
            for(int j=0; j<NUMVAR; ++j)
              task.putvartype(j,mosek.variabletype.type_int);
            task.putobjsense(mosek.objsense.maximize);

            try
              {
                task.optimize();
              }
            catch (mosek.Warning w)
              {
                Console.WriteLine("Mosek warning:");
                Console.WriteLine (w.Code);
                Console.WriteLine (w);
              }
            task.getsolutionslice(mosek.soltype.itg, /* Integer solution.       */
                                  mosek.solitem.xx,  /* Which part of solution.  */
                                  0,                 /* Index of first variable. */
                                  NUMVAR,            /* Index of last variable+1 */
                                  xx);                            

            for(int j = 0; j < NUMVAR; ++j)
              Console.WriteLine ("x[{0}]:{1}", j,xx[j]);
          }
        catch (mosek.Exception e)
            {
                Console.WriteLine (e.Code);
                Console.WriteLine (e);
            }

        if (task != null) task.Dispose ();
        if (env  != null)  env.Dispose ();
    }
}
