/* File:    qo1.cs

Purpose: Demonstrate how to solve a quadratic
optimization problem using the MOSEK .NET API.
*/

using System;

public class QuadOpt
{
  public static void Main ()
  {
    // Since the value infinity is never used, we define
    // 'infinity' symbolic purposes only
    const double infinity = 0;
    const int NUMCON = 1;   /* Number of constraints.             */
    const int NUMVAR = 3;   /* Number of variables.               */
    const int NUMANZ = 9;   /* Number of numzeros in A.           */
    const int NUMQNZ = 4;   /* Number of nonzeros in Q.           */

    double[] c = {0.0,-1.0,0.0};
    
    mosek.boundkey[]  bkc   = {mosek.boundkey.lo};
    double[] blc = {1.0};
    double[] buc = {infinity};
    
    mosek.boundkey[]  bkx   = {mosek.boundkey.lo,
                               mosek.boundkey.lo,
                               mosek.boundkey.lo};        
    double[] blx  = {0.0,
                     0.0,
                     0.0};
    double[] bux  = {+infinity,
                     +infinity,
                     +infinity};
     
    int[] ptrb = {0, 1, 2 };
    int[] ptre = {1, 2, 3};
    int[]    asub  = { 0,   0,   0};
    double[] aval  = {1.0, 1.0, 1.0};
        
    mosek.Task 
      task = null;
    mosek.Env
      env = null;
    double[] xx  = new double[NUMVAR];
    try
    {
      env  = new mosek.Env ();
      env.init ();
      task = new mosek.Task (env, NUMCON,NUMVAR);

      task.inputdata(NUMCON,NUMVAR,
                     c,0.0,
                     ptrb,
                     ptre,
                     asub,
                     aval,
                     bkc,
                     blc,
                     buc,
                     bkx,
                     blx,
                     bux);
      /*
       * The lower triangular part of the Q
       * matrix in the objective is specified.
       */

      int[]    qsubi = {0,   1,    2,   2  };
      int[]    qsubj = {0,   1,    0,   2  };
      double[] qval =  {2.0, 0.2, -1.0, 2.0};
            
      /* Input the Q for the objective. */

      task.putobjsense(mosek.objsense.minimize);

      task.putqobj(qsubi,qsubj,qval);

      task.optimize();

      task.getsolutionslice(mosek.soltype.itr,
                            mosek.solitem.xx,
                            0,
                            NUMVAR,
                            xx);
	    
      Console.WriteLine ("Primal solution");
      for ( int j=0; j<NUMVAR; ++j )
        Console.Write("x[{0}]: {1}\n",j,xx[j]);
    }
    catch (mosek.Exception e)
    {
      Console.WriteLine (e);
    }
    if (task != null) task.Dispose ();
    if (env  != null)  env.Dispose ();

  } /* Main */
}
