@echo off

set DOPAUSE=1
set DOCAPI=0
set DOJAVAAPI=0
set DODOTNETAPI=0
set IS64X86=0
set ISIA64=0

for %%i in ( %* ) do (
  if %%i==capi      set DOCAPI=1
  if %%i==javaapi   set DOJAVAAPI=1
  if %%i==dotnetapi set DODOTNETAPI=1
  if %%i==nopause   set DOPAUSE=0
  if %%i==64x86     set IS64X86=1
  if %%i==ia64      set ISIA64=1
)

set PLATFORM=win
set MOSEKNAME=mosek
set MOSEKLIBNAME=mosek
set MOSEKDOTNETDLL=mosekdotnet.dll

if %IS64X86%==1 (  
    set PLATFORM=win64x86
    set MOSEKNAME=mosek64
    set MOSEKLIBNAME=mosek64_
    set MOSEKDOTNETDLL=mosekdotnet64.dll
)

if %ISIA64%==1 (  
    set PLATFORM=winia64
    set MOSEKNAME=mosek64
    set MOSEKLIBNAME=mosek64_
    set MOSEKDOTNETDLL=mosekdotnet64.dll
)


@echo This script test the MOSEK optimization tools.
@echo     Platform     : %PLATFORM%
@echo Testing the interfaces (1=yes).
@echo     Test C API   : %DOCAPI% (capi)    
@echo     Test Java API: %DOJAVAAPI% (javaapi)
@echo     Test .NET API: %DODOTNETAPI% (dotnetapi)
@echo #
@echo #

if %DOPAUSE%==1 pause

set MOSEKBIN=.\platform\%PLATFORM%\bin

set OLDPATH=%PATH%
set PATH=%MOSEKBIN%;%PATH%

call mosekver

set MSKVER=%MOSEK_VER_MAJOR%_%MOSEK_VER_MINOR%

@echo MOSEK version: %MSKVER%

if NOT DEFINED MOSEKLM_LICENSE_FILE (
  @echo Using evaluation license file.
  set MOSEKLM_LICENSE_FILE=..\licenses
)

set CC=cl /MT -I .\platform\%PLATFORM%\h\ 

if %IS64X86%==1 (  
  set CC=%CC% bufferoverflowU.lib
)

if %ISIA64%==1 (  
  set CC=%CC% bufferoverflowU.lib
)

set MOSEK=.\platform\%PLATFORM%\bin\mosek -l ..\licenses
set MOSEKLIB=.\platform\%PLATFORM%\dll\%MOSEKLIBNAME%%MSKVER%.lib
set MOSEKDLLIB=.\platform\%PLATFORM%\lib\mosekdl.lib
set MOSEKDLL=.\platform\%PLATFORM%\bin\%MOSEKLIBNAME%%MSKVER%.dll
set MOSEKJAR=.\platform\%PLATFORM%\bin\mosek.jar

@echo Testing the MOSEK installation on the Windows platform
@echo #
@echo #

@echo Showing version information.
%MOSEK% -f

@echo #
@echo #
@echo Solving a problem.

@echo Testing interior-point optimizer
%MOSEK% examples\data\lo1.mps

@echo #
@echo #

@echo Testing primal simplex optimizer
%MOSEK% examples\data\lo1.mps -d MSK_IPAR_OPTIMIZER MSK_OPTIMIZER_PRIMAL_SIMPLEX

@echo #
@echo #

@echo Testing mixed integer optimizer
%MOSEK%  examples\data\flugpl.mps

@echo #
@echo #

if %DOCAPI%==1 (

  @echo Testing C API
  @echo   If you do not have Microsoft C installed
  @echo   then this test FAILS.
  @echo #
  @echo #

  @echo Linking in the Mosek library. LO examples.
  %CC% examples\c\lo1.c %MOSEKLIB%
  lo1
  @echo #
  @echo #

  @echo Linking in the Mosek library. QO examples.
  %CC% examples\c\qo1.c %MOSEKLIB%
  qo1
  @echo #
  @echo #

  @echo Linking in the Mosek library. MILO examples.
  %CC% examples\c\milo1.c %MOSEKLIB%
  milo1
  @echo #
  @echo #

  @echo Linking in the Mosek library. CQP examples.
  %CC% examples\c\cqo1.c %MOSEKLIB%
  cqo1
  @echo #
  @echo #

  @echo Linking in the Mosek library. NLO examples.
  %CC% examples\c\nlo1.c %MOSEKLIB%
  nlo1
  @echo #
  @echo #

  @echo Linking in the Mosek library. dgopt optimizer.
  %CC% examples\c\mskdgopt.c examples\c\dgopt.c %MOSEKLIB% /Ferundgopt
  rundgopt examples\data\dgo.mps examples\data\dgo.f
  @echo #
  @echo #

  @echo Linking in the Mosek library. expopt optimizer.
  %CC% examples\c\tstexpopt.c examples\c\scopt.c examples\c\expopt.c examples\c\dgopt.c %MOSEKLIB%
  tstexpopt
  @echo #
  @echo #

  @echo Linking in the Mosek library. scopt optimizer.
  %CC% examples\c\tstscopt.c examples\c\scopt.c %MOSEKLIB%
  tstscopt
  @echo #
  @echo #
  @echo Finished C API tests
  

  if %DOPAUSE%==1 pause          
) 

if %DOJAVAAPI%==1 (

  @echo Testing the JAVA API
  @echo   If you do not have JAVA installed 
  @echo   then this test FAILS. 
  @echo #
  @echo #
  @echo   Compiling Java code.  
  @echo      This  requires the 'javac' command.

  if %DOPAUSE%==1 pause

  mkdir tmp\java

  for %%i in ( lo1 lo2 qo1 cqo1  ) do ( 

      @echo # Java example: %%i 
      
      call javac -d .\tmp\java -classpath %MOSEKJAR% examples\java\\%%i.java 
      
      java -classpath %MOSEKJAR%;.\tmp\java   %%i.%%i

  )

  @echo done testing java
)


if %DODOTNETAPI%==1 (

  @echo Testing .NET

  @echo #
  @echo #

  copy platform\%PLATFORM%\bin\%MOSEKDOTNETDLL%  .

  @echo Testing C# .NET
  csc /r:%MOSEKDOTNETDLL% /out:cs-lo1.exe examples\dotnet\lo1.cs 
  cs-lo1
  @echo #
  @echo #

  csc /r:%MOSEKDOTNETDLL% /out:cs-qo1.exe examples\dotnet\qo1.cs 
  cs-qo1
  @echo #
  @echo #

  @echo Testing Visual basic .NET 
  vbc /r:%MOSEKDOTNETDLL% /out:vb-lo1.exe examples\dotnet\lo1.vb
  vb-lo1
  @echo #
  @echo #
  erase %MOSEKDOTNETDLL%

  @echo done testing .NET
  @echo #
  @echo #
)

@echo *** Checking FLEXlm version
@echo   lmgrd:
platform\%PLATFORM%\bin\lmutil lmver platform\%PLATFORM%\bin\lmgrd 
platform\%PLATFORM%\bin\lmutil lmver %MOSEKDLL%
@echo #
@echo #

@echo *** Checking MOSEK version
platform\%PLATFORM%\bin\mosek -x -f
@echo #
@echo #


set PATH=%OLDPATH%

 
